#include "../includes/main.h"

/*----------initialisations ----------*/

// Initialise une liste de devis
ListeDevis newListeDevis(void)
{
	trace("newListeDevis");
	return NULL;
}

// Initialise une nouvelle offre
Offre *newOffre(char *travauxName)
{
	trace("newOffre");
	Offre *o;
	o = (Offre *)malloc(sizeof(Offre));
	strcpy(o->travaux, travauxName);
	o->ldevis=newListeDevis();
	return o;
}

// Initialise une liste (pour les successeurs)
Liste newListe(void)
{
	trace("newListe");
	return NULL;
}

/*------------verifications------------*/

// Verifie l'integrite d'une liste de devis
Booleen emptyListe(ListeDevis l)
{
	trace("emptyListe");
	return l==NULL;
}

// Vérifie l'intégrité de l'offre
Booleen emptyOffre(Offre *o)
{
	trace("emptyOffre");
	return emptyListe(o->ldevis);
}

/*-------------insertions---------------*/

// Insere un devis en tete de la liste des devis 
ListeDevis insert(ListeDevis l, Devis d)
{
	trace("insert");
	MaillonDevis *md;
	md = (MaillonDevis *)malloc(sizeof(MaillonDevis));
	if (md == NULL)
	{
		printf("\033[0;31mErreur: \033[0mallocation mémoire insertion MaillonDevis\n");
		exit(1);
	}
	md->dev=d;
	md->suiv=l;
	return md;
}

// Insere en queue de liste (les successeurs)

Liste insertSucc(Liste l, char travaux[])
{
	trace("insertSucc");
	MaillonSucc *ms;
	ms = (MaillonSucc *)malloc(sizeof(MaillonSucc));
	if (ms == NULL)
	{
		printf("\033[0;31mErreur: \033[0mmalloc liste successeurs\n");
		exit(1);
	}
	ms->nxt=l->nxt;
	l->nxt=ms;
	strcpy(ms->tache, travaux);
	return l;
}

/*------------suppressions-------------*/

// Supprime le premier maillon de la liste de devis
ListeDevis del(ListeDevis l)
{
	trace("del");
    MaillonDevis *md;
    if (l == NULL)
    {
        printf("\033[0;31mErreur: \033[0msuppression dans une liste vide\n");
        exit(1);
    }

    md = l;
    l = l->suiv;

    if (md->suiv == NULL)
    {
        free(md->dev.nomE);
        free(md->dev.adresse);
        free(md);
        return newListeDevis();
    }

    free(md->dev.nomE);
    free(md->dev.adresse);
    free(md);
    return l;
}



/*---------------calculs----------------*/

// Calcule la longueur de la liste de devis
int lenListeDevis(ListeDevis l)
{
	trace("lenListeDevis");
	int len=0;
	while (l)
	{
		len++;
		l=l->suiv;
	}
	return len;
}

int lenMaxNomEntreprise(ListeDevis ldevis) 
{
	trace("lenMaxNomEntreprise");
	int maxLen;
	int len;

	maxLen = 0;
	while (ldevis) 
	{
		len = strlen(ldevis->dev.nomE);
		if (len > maxLen) 
			maxLen = len;
		ldevis = ldevis->suiv;
	}
	return maxLen;
}

// Calcule le nombre de precedences pour un travail donne
int nombrePred(char travaux[], Precedence prec[], int tlog)
{
	trace("nombrePred");
	int nbPred = 0, i;
	for (i=0; i<tlog; i++)
		if (strcmp(travaux, prec[i].travauxSucc)==0)
			nbPred++;
	return nbPred;
}

/*--------------informations------------*/

// Retourne une liste des successeurs pour un travail donne
Liste lstSucc(char travaux[], Precedence prec[], int tlog)
{
	trace("lstSucc");
	Liste succ = newListe();
	int i;
	for (i=0; i<tlog; i++)
		if(strcmp(travaux, prec[i].travauxPrec)==0)
			succ=insertSucc(succ, prec[i].travauxSucc);
	return succ;
}

/*---------------affichages--------------*/

// Affiche les details d'un devis
void displayDevis(Devis d)
{
	trace("displayDevis");
	printf("\t%s\t%s\t%d\t%d\t%d\n", d.nomE, d.adresse, d.capital, d.duree, d.prix);
}

// Affiche les details de l'offre et sa liste de devis
void displayOffre(Offre *o)
{
	trace("displayOffre");
	printf("\n%s\n", o->travaux);
	displayListeDevis(o->ldevis);
}

// Affiche les details de la liste de devis
void displayListeDevis(ListeDevis l)
{
	trace("displayListeDevis");
	while(l != NULL)
	{
		displayDevis(l->dev);
		l=l->suiv;
	}
}

// Fonction pour afficher le devis d'une entreprise donnée pour un type de travaux donné
void afficherDevisEntreprise(Offre **tabTravaux)
{
	trace("afficherDevisEntreprise");
	int indiceTravaux;
	int maxlen;
	char typeTravaux[30];
	char *nomEntreprise;

	printf("Saisir le type de travaux : ");
	fgets(typeTravaux, sizeof(typeTravaux), stdin);
	typeTravaux[strlen(typeTravaux) - 1] = '\0';

	indiceTravaux = rechTravaux(typeTravaux, tabTravaux);

	if (indiceTravaux == -1) 
	{
		printf("Type de travaux non trouvé.\n");
		return;
	}

	maxlen = lenMaxNomEntreprise(tabTravaux[indiceTravaux]->ldevis);
	nomEntreprise = malloc((maxlen + 1) * sizeof(char));
	printf("Saisir le nom de l'entreprise : ");
	fgets(nomEntreprise, maxlen + 1, stdin);

	ListeDevis listeDevis = tabTravaux[indiceTravaux]->ldevis;
	while (listeDevis) 
	{
		if (strcmp(listeDevis->dev.nomE, nomEntreprise) == 0) 
		{
			printf("Devis pour %s de l'entreprise %s :\n", typeTravaux, nomEntreprise);
			displayDevis(listeDevis->dev);
			return;
		}
		listeDevis = listeDevis->suiv;
	}

	printf("Devis non trouvé pour l'entreprise %s dans le type de travaux %s.\n", nomEntreprise, typeTravaux);
	free(nomEntreprise);
}

void displayPrecedences(Precedence *tabP, int tlog) 
{
	trace("displayPrecedences");
    printf("Liste des precedences :\n");

    for (int i = 0; i < tlog; i++) {
        printf("\t%s\t\t : \t%s\n", tabP[i].travauxPrec, tabP[i].travauxSucc);
    }
}


// Partie 4 (à trier)


